//**********************************************************************************************************
// author	Tony Madden	(tm@mqware.com)
//**********************************************************************************************************
#include	<maxconn.h>

#include	<x_exception.hpp>
#include	<x_argumentparser.hpp>
#include	<x_mq.hpp>
#include	<maxconnworkset.hpp>

#include	<string>
#include	<exception>
#include	<sstream>
#include	<iostream>

#include	<imqi.hpp> 
#include	<cmqc.h>
#include	<cmqxc.h> 


using namespace std;

#define	MXC_UTIL_PROG_IDENT	"maxconnutil v1.0.1"


ImqQueueManager*	pQM;

X_ArgumentParser*	pMainParameters;
X_ArgumentParser*	pFileParameters;

MaxConnWorkSet*		pWorkSet;
MaxConnStateData*	pStateData;

long				lState;
bool				bClearAllMessages;
string				sModelQueueName;
string				sQMgrName;



void	allVersions();
bool	browseMessage(const char*,ImqMessage*);
void	clearMessages();
void	closeQueue();
void	connectToQMgr();
void	createQueue();
void	displayState();
void	displayStateQueueMessage( ImqMessage*);
string	dottedOutput(string s1 );
bool	initialize(int,char**);
void	loadQueue();
bool	openQueue();
void	printHelp();
void	printout(string);
void	loadState();
void	createChannelDefState();
void	retrieveChannelDefParameters(X_ArgumentParser*);
void	shutdown();


int main(int arg_c, char **arg_v) 
{
	int rc=0;

	printout(MXC_UTIL_PROG_IDENT );

	if (arg_c < 2|| arg_v[1] == "?"){
		printHelp();
		return 0;
	}

	try{
		if (!initialize(arg_c,arg_v)){
			return 0;
		}

		// This program can either load a State queue or display the contents...
		if (lState == MXC_STATE_LOAD){
			loadState();
		}else{
			displayState();
		}

		shutdown();
	}catch(X_Exception x){
		string err = "Ended with error: " + x.toString();
		printout(err);
	}catch(...){
		printout("???");
	}

	return 0;
}

//**********************************************************************************************************
// Initialize the program environment using the main arguments
//
// arg_c and arg_v			main arguments
//
// return					true if successful initialization
//**********************************************************************************************************
bool	initialize(int arg_c, char **arg_v){

	bool bRc = true;

	pFileParameters = 0;

	bClearAllMessages = false;

	sModelQueueName = MXC_DEFAULT_MODEL_QUEUE_NAME;

	pStateData = new MaxConnStateData();

	pWorkSet = new MaxConnWorkSet();

	// Process the main arguments
	pMainParameters = new X_ArgumentParser(_AP_SWITCHES,_AP_COMMENT);
	pMainParameters->loadArgumentsFromMain(arg_c,arg_v);
	pMainParameters->parse(true,true);

	if (pMainParameters->numberOfInstances(MXC_PARM_HELP)){
		printHelp();
		bRc = false;
	}else{
		lState = 0;
		// Program must be called for either load or display
		if (pMainParameters->numberOfInstances(MXC_PARM_LOAD) && pMainParameters->numberOfInstances(MXC_PARM_DISPLAY)){
			throw X_Exception(E_ERROR,0,"Error - Only one of Load and Display arguments can be specified");
		}else{
			if (!(pMainParameters->numberOfInstances(MXC_PARM_LOAD)) && !(pMainParameters->numberOfInstances(MXC_PARM_DISPLAY))){
				throw X_Exception(E_ERROR,0,"Error - Either Load or Display argument must be specified");
			}
		}

		string s1;
		if (pMainParameters->numberOfInstances(MXC_PARM_LOAD)){
			// Load a State queue
			lState = MXC_STATE_LOAD;
			string sFileName;
			// Check that there is a parameter file
			if (pMainParameters->get(MXC_PARM_PARAMETER_FILE,&sFileName)){
				// Parse the contents of the file
				pFileParameters = new X_ArgumentParser(_AP_NAME_VALUE_PAIRS,_AP_COMMENT);
				pFileParameters->loadArgumentSetsFromFile(sFileName,MXC_PARAMETER_FILE_SET_ARGUMENT_NAME);
				pFileParameters->parse(true,true);
			}else{
				throw X_Exception(E_ERROR,0,"Error - No Parameter File value was specified");
			}
			printout("Load State Queues\n");
		}else{
			// Display contents of a State queue
			lState = MXC_STATE_DISPLAY;
			s1 = "";
			if (!pMainParameters->get(MXC_PARM_SVRCONN_CHANNEL_NAME,&s1)){
				throw X_Exception(E_ERROR,0,"Error - ServerConnection channel name must be provided");
			}
			pWorkSet->setChannelName(s1);

			// A state queue can be specified by name 
			// Normally, the name is derived from the SVRCONN channelname  (i.e. "MAXCONN."[svrconn channel name])
			if (pMainParameters->get(MXC_PARM_QUEUE_NAME,&s1)){
				pWorkSet->setQName(s1);
			}else{
				pWorkSet->setQName(MXC_QUEUE_NAME_PREFIX + pWorkSet->getChannelName());
			}
			ostringstream ost;
			ost << "Display State Queue for SvrConn Channel " << pWorkSet->getChannelName() << endl;
			printout(ost.str());
			
		}

		if (pMainParameters->get(MXC_PARM_QMGR_NAME,&sQMgrName)){
			printout("Queue Manager name is " + sQMgrName);
		}

		// By default, each State queue only contains messages for one SVRCONN channel  (This isn't necessarily the case - but 
		// is strongly recommended !! ).
		// If a state queue has state messages for more than one channel (bad idea), then MXC_PARM_ONLY_CLEAR_SPECIFIC_MESSAGES will 
		// result in the clear down of only thosestate messages for channels which are being defined.
		if (pMainParameters->numberOfInstances(MXC_PARM_ONLY_CLEAR_SPECIFIC_MESSAGES)){
			bClearAllMessages = false;
		}else{
			// much better idea. Have one state queue per channel, and clear it down totally before (re-)loading
			bClearAllMessages = true;
		}

		// Check to see whether any invalid arguments have been entered.
		if (pMainParameters->isUnusedNames()){
			vector<string>	vs;
			s1 = "";
			pMainParameters->getUnusedNames(&vs);
			for (int i = 0;i<vs.size();i++){
				s1 += vs[i] + " ";
			}

			ostringstream	ost;
			if (lState == MXC_STATE_LOAD){
				ost << "Error - The following arguments are illegal for a load operation: " << s1;
				throw X_Exception(E_ERROR,0,ost.str());
			}else{
				ost << "Error - The following arguments are illegal for a display operation: " << s1;
				throw X_Exception(E_ERROR,0,ost.str());
			}
		}

		// Connect to the qmgr
		connectToQMgr();

		string s = "QMgr is ";
		s += (char*)pQM->name();
		printout(s);
	}

	return bRc;
}

//**********************************************************************************************************
// Load a State queue
//**********************************************************************************************************
void	loadState(){

	vector<string> parameterSets;
	string			sChannelDef;

	// parameterSets will be loaded with strings - each string containing a set of Connection Definition parameters
	// See the documentation
	pFileParameters->get(MXC_PARAMETER_FILE_SET_ARGUMENT_NAME,&parameterSets);

	X_ArgumentParser* pChannelParameters = new X_ArgumentParser(_AP_NAME_VALUE_PAIRS,_AP_COMMENT);

	ostringstream	ost;
	ost << parameterSets.size() << " ChannelDefinitions will be processed\n";
	printout(ost.str());

	// Iterate thru the parameter sets
	for (int i=0;i < parameterSets.size();i++){
		printout("*******************************************************");
		ostringstream	ost;
		ost << "Processing ChannelDefinition " << (i+1) << " ..." << endl;
		printout(ost.str());
		sChannelDef = parameterSets[i];
		pChannelParameters->reset();
		pWorkSet->reset();
		pStateData->reset();
		// Load a set of Connection Definition parameters into the parser
		pChannelParameters->loadArgumentsFromString(sChannelDef);
		pChannelParameters->parse(true,true);
		retrieveChannelDefParameters(pChannelParameters);
		// Create State info for the channel in question
		createChannelDefState();
		printout(" ");
	}

}

//**********************************************************************************************************
// Display the contents of a State queue

//**********************************************************************************************************
void	displayState(){

	// State messages are keyed on CorrelationId...
	pWorkSet->setBootstrapCorrelationId( (char*)(MXC_BOOTSTRAP_PREFIX + pWorkSet->getChannelName()).c_str(),(MXC_BOOTSTRAP_PREFIX + pWorkSet->getChannelName()).size());
	pWorkSet->setVolatileCorrelationId( (char*)(MXC_VOLATILE_PREFIX + pWorkSet->getChannelName()).c_str(),(MXC_BOOTSTRAP_PREFIX + pWorkSet->getChannelName()).size());

	pWorkSet->setOpenType(MXC_BROWSE);

	printout("State Queue name is " + pWorkSet->getQName());

	if (!openQueue()){
		ostringstream	ost;
		ost << "State queue " <<  pWorkSet->getQName() << " does not exist";
		throw X_Exception(E_ERROR,0,ost.str());
	}

	ImqMessage*	pmd = new ImqMessage();
	bool	bFirst = true;

	while(true){
		// Get Volatile message (note that allowance is made for multiple Bootstrap messages - this would be an error in State setup though !)
		if (!browseMessage(pWorkSet->getBootstrapCorrelationId(),pmd)){
			if (bFirst){
				throw X_Exception(E_ERROR,0,"Error - Bootstrap message does not exist");
			}else{
				break;
			}
		}
		if (bFirst){
			bFirst = false;
		}else{
			ostringstream	ost;
			ost << endl << " ERROR - State queue has multiple Bootstrap messages for channel " << pWorkSet->getChannelName() << endl;
			printout(ost.str());
		}
		printout(" ");
		printout("BOOTSTRAP Message Contents:");
		// display Bootstrap State info
		displayStateQueueMessage(pmd);
	}

	bFirst = true;
	while(true){
		// Get Volatile message (note that allowance is made for multiple Volatile messages - this would be an error in maxconn processing though !)
		if (!browseMessage(pWorkSet->getVolatileCorrelationId(),pmd)){
			if (bFirst){
				printout(" ");
				printout("Volatile Message does not exist");
			}
			break;
		}
		if (bFirst){
			bFirst = false;
		}else{
			ostringstream	ost;
			ost << endl << " ERROR - State queue has multiple Volatile messages for channel " << pWorkSet->getChannelName() << endl;
			printout(ost.str());
		}
		printout(" ");
		printout("VOLATILE Message Contents:");
		// Display Volatile State info
		displayStateQueueMessage(pmd);
	}

	closeQueue();

}

//**********************************************************************************************************
// Shutdown
//**********************************************************************************************************
void	shutdown(){
	
	pQM->disconnect();
	if (pQM->reasonCode() != MQRC_NONE  ){
		ostringstream	ost;
		ost << "Error - MQDisconnect failed with reason " << X_Mq::convertToStringMQRC(pQM->reasonCode());
		throw X_Exception (E_ERROR,0,ost.str());
	}
}


//**********************************************************************************************************
// Process an individual Connection Definition
//
// arg_pParameters					object containing startup parameters
//**********************************************************************************************************
void	retrieveChannelDefParameters(X_ArgumentParser* arg_pParameters ){
	string	s1;

	if (!arg_pParameters->get(MXC_PARMFILE_SVRCONN_CHANNEL_NAME,&s1) || s1.size() == 0){
		throw X_Exception(E_ERROR,0,"Error - No ServerConnection channel value was specified");
	}
	pWorkSet->setChannelName(s1);
	pWorkSet->setQName(MXC_QUEUE_NAME_PREFIX + pWorkSet->getChannelName());
	pWorkSet->setBootstrapCorrelationId( (char*)(MXC_BOOTSTRAP_PREFIX + pWorkSet->getChannelName()).c_str(),(MXC_BOOTSTRAP_PREFIX + pWorkSet->getChannelName()).size());
	pWorkSet->setVolatileCorrelationId( (char*)(MXC_VOLATILE_PREFIX + pWorkSet->getChannelName()).c_str(),(MXC_BOOTSTRAP_PREFIX + pWorkSet->getChannelName()).size());

	// Set up log file name
	if (arg_pParameters->get(MXC_PARMFILE_LOG_FILE_NAME,&s1)){
		pStateData->setLogFileName( s1);
	}else{
		ostringstream	ost;
		ost << MXC_DEFAULT_LOG_FILE_PREFIX << '.' << (char*)pQM->name() << '.' << pWorkSet->getChannelName() << '.' << MXC_DEFAULT_LOG_FILE_SUFFIX;
		pStateData->setLogFileName( ost.str());
	}

	// Set up log file directory
	if (arg_pParameters->get(MXC_PARMFILE_LOG_FILE_DIRECTORY,&s1)){
		pStateData->setLogFileDirectory( s1);
	}else{
		pStateData->setLogFileDirectory( MXC_DEFAULT_LOG_FILE_DIRECTORY);
	}

	// Set trace level
	if (arg_pParameters->get(MXC_PARMFILE_TRACE_LEVEL,&s1) ){
		if (X_String::toLower(s1) == MXC_TRACE_LEVEL_DEBUG_TEXT){
			pStateData->setTraceLevel(MXC_TRACE_LEVEL_DEBUG);
		}else{
			if (X_String::toLower(s1) == MXC_TRACE_LEVEL_INFO_TEXT){
				pStateData->setTraceLevel( MXC_TRACE_LEVEL_INFO);
			}else{
				
				if (X_String::toLower(s1) == MXC_NO){
					pStateData->setTraceLevel(0);
				}else{
					ostringstream	ost;
					ost << "Error - Trace level parameter is invalid: " << s1;
					throw X_Exception(E_ERROR,0,ost.str());
				}
			}
		}
	}

	// Set max number of connections
	if (arg_pParameters->get(MXC_PARMFILE_MAX_NUMBER_OF_CONNECTIONS,&s1)){
		if (X_String::isNumeric(s1)){
			pStateData->setMaxConnections(atol(s1.c_str()));
			if (pStateData->getMaxConnections() < 1){
				throw X_Exception(E_ERROR,0,"Error - Max number of connections is zero. ");
			}
		}else{
			ostringstream	ost;
			ost << "Error - Max number of connections parameter is not numeric: " << s1;
			throw X_Exception(E_ERROR,0,ost.str());
		}
	}else{
		throw X_Exception(E_ERROR,0,"Error - No MaxNumberOfConnections value was specified");
	}

	// Set max number of in use attempts
	if (arg_pParameters->get(MXC_PARMFILE_MAX_NUMBER_OF_IN_USE_ATTEMPTS,&s1)){
		if (X_String::isNumeric(s1)){
			pStateData->setMaxNumberOfInUseAttempts (atol(s1.c_str()));
			if (pStateData->getMaxNumberOfInUseAttempts() < 1){
				throw X_Exception(E_ERROR,0,"Error - Max number of in use attempts parameter is zero. ");
			}
		}else{
			ostringstream	ost;
			ost << "Error - Max number of in use attempts parameter is not numeric: " << s1;
			throw X_Exception(E_ERROR,0,ost.str());
		}
	}else{
		pStateData->setMaxNumberOfInUseAttempts(MXC_DEFAULT_MAX_OBJECTS_IN_USE_ATTEMPTS);
	}

	// Set frequenct of pcf check for actual number of connections (default - every time max connections is reached)
	if (arg_pParameters->get(MXC_PARMFILE_FREQUENCY_OF_PCF_CHECKS,&s1)){
		// NOTE - this argument can be zero
		if (X_String::isNumeric(s1)){
			pStateData->setFrequencyOfPCFChecks(atol(s1.c_str()));;
		}else{
			ostringstream	ost ;
			ost << "Error - Frequency of PCF checks is not numeric: " << s1;
			throw X_Exception(E_ERROR,0,ost.str());
		}
	}else{
		pStateData->setFrequencyOfPCFChecks(1);
	}

	//set initial event state (whether used or not)
	pStateData->setEventState(MXC_EVENT_STATE_OK);
	// Event flag
	if (arg_pParameters->get(MXC_PARMFILE_EVENT_FLAG,&s1)){
		if (X_String::equalsIgnoreCase(s1,MXC_LIT_EVENTS_ACTIVE)){
			pStateData->setEventFlag( pStateData->getEventFlag() | MXC_EVENTS_ACTIVE);
		}else{
			if (X_String::equalsIgnoreCase(s1,MXC_LIT_EVENTS_MQ_ACTIVE)){
				pStateData->setEventFlag( pStateData->getEventFlag() | MXC_EVENTS_MQ_ACTIVE);
	
				if (arg_pParameters->get(MXC_PARMFILE_EVENT_QUEUE_NAME,&s1)){
					pStateData->setEventQueue(s1);
				}else{
					pStateData->setEventQueue(MXC_DEFAULT_EVENT_QUEUE_NAME);
				}
			}
		}
	}

	// Set warn threshold, but only if events are enabled
	if (pStateData->getEventFlag()){
		if (arg_pParameters->get(MXC_PARMFILE_EVENT_WARN_THRESHOLD,&s1)){
			// NOTE - this argument must be less than max connections
			if (X_String::isNumeric(s1)){
				pStateData->setWarnThreshold(atol(s1.c_str()));
				if (pStateData->getWarnThreshold() >= pStateData->getMaxConnections() || pStateData->getWarnThreshold() < 1){
					throw X_Exception(E_ERROR,0,"Error - Event warn level must be less than max connections and greater than zero");
				}
			}else{
				throw X_Exception(E_ERROR,0,"Error - Event warn level is not numeric");
			}
		}
	}else{
		// If there is no warn level parameter, then de-activate warn processing.
		pStateData->setWarnThreshold(0);
	}

	printout("SvrConn channel name is " + pWorkSet->getChannelName());
	
	printout("Queue name is " + pWorkSet->getQName());

	{
		ostringstream	ost;
		ost << "Max Number of Connections is " << pStateData->getMaxConnections();
		printout(ost.str()); 
	}

	{
		ostringstream	ost;
		ost << "Max Number of In Use Attempts is " << pStateData->getMaxNumberOfInUseAttempts();
		printout(ost.str()); 
	}

	{
		ostringstream	ost;
		ost << "Frequency of PCF checks is " << pStateData->getFrequencyOfPCFChecks();
		printout(ost.str()); 
	}

	printout("LogFile Directory is " + pStateData->getLogFileDirectory());

	printout("LogFile Name is " + pStateData->getLogFileName());

	if (pStateData->getTraceLevel()){
		ostringstream	ost;
		ost << "TraceLevel is ";
		switch ((int)pStateData->getTraceLevel()){
			case MXC_TRACE_LEVEL_INFO:
				ost << MXC_TRACE_LEVEL_INFO_TEXT;
				break;
			case MXC_TRACE_LEVEL_DEBUG:
				ost << MXC_TRACE_LEVEL_DEBUG_TEXT;
				break;
			default:
				ost << pStateData->getTraceLevel() << " (unknown value)";
		}
		printout(ost.str());
	}else{
		printout("Trace logging is inactive");
	}

	if (pStateData->getEventFlag()){
		printout("Event reporting is active");
		if ((pStateData->getEventFlag() & MXC_EVENTS_MQ_ACTIVE) == MXC_EVENTS_MQ_ACTIVE){
			printout("Event reporting to event queue is active.");
			printout("Event Queue is " + pStateData->getEventQueue());
		}else{
			printout("Event reporting to event queue is inactive");
		}

	}
	
}

//**********************************************************************************************************
// Parse and display information from a State message
//
// arg_pMd							MQ message object
//**********************************************************************************************************
void	displayStateQueueMessage( ImqMessage* arg_pMd){
	
	long	lValueOffset = 50;

	string sDots = "........................";
	string s1;

	pStateData->loadStateData(arg_pMd->dataPointer(),arg_pMd->dataLength());

	s1 = dottedOutput("StateDataType") + pStateData->getStateDataType();
	printout(s1);

	{
		ostringstream	ost;
		ost <<  dottedOutput("MaxConnections") << pStateData->getMaxConnections();
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("ExistingConnections") << pStateData->getExistingConnections();
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("MaxNumberOfInUseAttempts") << pStateData->getMaxNumberOfInUseAttempts();
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("TimesAtConnMaxSinceLastPCFCheck")  << pStateData->getTimesAtConnMaxSinceLastPCFCheck();
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("FrequencyOfPCFChecks") << pStateData->getFrequencyOfPCFChecks();
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("EventFlag") << pStateData->getEventFlag() << "  (";
		if (pStateData->getEventFlag() == MXC_EVENTS_NOT_ACTIVE){
			ost << "NOT ACTIVE";
		}else{
			if ((pStateData->getEventFlag() & MXC_EVENTS_MQ_ACTIVE) == MXC_EVENTS_MQ_ACTIVE){
				ost << "MQ ACTIVE";
			}else{
				ost << "ACTIVE";
			}
		}
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("WarnThreshold") << pStateData->getWarnThreshold();
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("EventState") << pStateData->getEventState() << "  (";
		if (pStateData->getEventState() == MXC_EVENT_STATE_OK){
			ost << "OK" ; 
		}else{
			if (pStateData->getEventState() == MXC_EVENT_STATE_WARN){
				ost << "WARN" ; 
			}else{
				if (pStateData->getEventState() == MXC_EVENT_STATE_MAX){
					ost << "MAX" ; 
				}else{
					ost << "unknown";
				}
			}
		}
		ost << ")";
		printout(ost.str());
	}

	{
		ostringstream	ost;
		ost << dottedOutput("TraceLevel") << pStateData->getTraceLevel() << "  (";
		if (pStateData->getTraceLevel() == MXC_TRACE_LEVEL_NONE){
			ost << "NONE" ; 
		}else{
			if (pStateData->getTraceLevel() == MXC_TRACE_LEVEL_INFO){
				ost << "INFO" ; 
			}else{
				if (pStateData->getTraceLevel() == MXC_TRACE_LEVEL_DEBUG){
					ost << "DEBUG" ; 
				}else{
					ost << "unknown";
				}
			}
		}
		ost << ")";
		printout(ost.str());
	}

	s1 = dottedOutput("LogFileDirectory") + pStateData->getLogFileDirectory();
	printout(s1);

	s1 = dottedOutput("LogFileName") + pStateData->getLogFileName();
	printout(s1);

	s1 = dottedOutput("EventQueue") + pStateData->getEventQueue();
	printout(s1);
}


//**********************************************************************************************************
// Open the relevant q, load it, close it
//**********************************************************************************************************
void	createChannelDefState(){

	pWorkSet->setOpenType( MXC_GET_PUT);
	if (openQueue() ){
		// Remove existing messages from the queue (not necessarily all - see clearMessages() )prior to loading it
		clearMessages();
	}
	loadQueue();
	closeQueue();

}

//**********************************************************************************************************
// Connect to a qmgr
//**********************************************************************************************************
void	connectToQMgr(){
	
	pQM = new ImqQueueManager();

	if (sQMgrName.size()){
		pQM->setName(sQMgrName.c_str());
	}

	pQM->connect();
	if (pQM->reasonCode() != MQRC_NONE){
		throw X_Exception ("MQConnect to " + sQMgrName + " failed with reason " + X_Mq::convertToStringMQRC(pQM->reasonCode()));
	}
}

//**********************************************************************************************************
// Open a maxconn state queue 
// If the queue doesn't exist then it is created 
//
// return			true if queue already exists, false if queue needed creating
//**********************************************************************************************************
bool	openQueue(){
	bool	bExtant = true;

	pWorkSet->getQueue()->setConnectionReference( *pQM);

	pWorkSet->getQueue()->setName(pWorkSet->getQName().c_str());

	if (pWorkSet->getOpenType() == MXC_BROWSE){
		pWorkSet->getQueue()->setOpenOptions(MQOO_BROWSE);
	}else{
		pWorkSet->getQueue()->setOpenOptions(MQOO_INPUT_EXCLUSIVE|MQOO_OUTPUT);
	}

	for (int i = 0; ;i++){
		pWorkSet->getQueue()->open();
		if (pWorkSet->getQueue()->reasonCode() == MQRC_NONE  ){
			break;
		}else{
			if (pWorkSet->getQueue()->reasonCode() == MQRC_UNKNOWN_OBJECT_NAME  ){
				// Queue doesn't exist so create it
				createQueue();
				bExtant = false;
				break;
			}else{
				if (pWorkSet->getQueue()->reasonCode() == MQRC_OBJECT_IN_USE  ){
					// The queue is in use (exclusive) so loop round a few times...
					if (i < pStateData->getMaxNumberOfInUseAttempts()){
						continue;
					}else{
						ostringstream	ost;
						ost << "MQOpen for queue " << pWorkSet->getQName() << " returned MQRC_OBJECT_IN_USE for max attempts count of " << pStateData->getMaxNumberOfInUseAttempts();
						throw X_Exception (E_ERROR,0,ost.str());
					}
				}else{
					ostringstream	ost;
					ost << "MQOpen for queue " << pWorkSet->getQName() << " failed with reason " <<X_Mq::convertToStringMQRC(pWorkSet->getQueue()->reasonCode());
					throw X_Exception (E_ERROR,0,ost.str());
				}
			}
		}
	}

	return bExtant;
}

//**********************************************************************************************************
// Create a state queue
//**********************************************************************************************************
void	createQueue(){

	pWorkSet->getQueue()->setConnectionReference( *pQM);
	// NOTE - make sure that the model queue has a DEFTYPE of PERMDYN
	pWorkSet->getQueue()->setName(sModelQueueName.c_str());
	pWorkSet->getQueue()->setDynamicQueueName(pWorkSet->getQName().c_str());
	pWorkSet->getQueue()->setOpenOptions(MQOO_INPUT_EXCLUSIVE|MQOO_OUTPUT);

	pWorkSet->getQueue()->open();
	if (pWorkSet->getQueue()->reasonCode() != MQRC_NONE  ){
		ostringstream	ost;
		ost << "MQOpen for model queue " << sModelQueueName << " failed with reason " << X_Mq::convertToStringMQRC(pWorkSet->getQueue()->reasonCode());
		throw X_Exception (E_ERROR,0,ost.str());
	}

	printout("Queue "+pWorkSet->getQName() + " did not exist and has been created");
}

//**********************************************************************************************************
// Close the queue
//
//**********************************************************************************************************
void	closeQueue(){

	pWorkSet->getQueue()->close();
	if (pWorkSet->getQueue()->reasonCode() != MQRC_NONE){
		ostringstream	ost;
		ost << "MQClose for queue " << pWorkSet->getQName() << " failed with reason " << X_Mq::convertToStringMQRC(pWorkSet->getQueue()->reasonCode()); 
		throw X_Exception (E_ERROR,0,ost.str());
	}

}

//**********************************************************************************************************
// Remove messages from state queue
//
//**********************************************************************************************************
void	clearMessages(){
	
	ImqGetMessageOptions* pgmo = new ImqGetMessageOptions();
	ImqMessage*	pmd = new ImqMessage();
	int		iLoop;

	// ClearAllMessages means remove every message from the queue
	// !ClearAllMessages means only remove those messages with MsgId and CorrelId relevant to the channel in question
	if (bClearAllMessages){
		iLoop = 1;
		pgmo->setMatchOptions( MQMO_NONE);
	}else{
		iLoop = 2;
		// Match correlid in case multiple State queues are defined as aliases which resolve to the same local q (NOT RECOMMENDED)
		pgmo->setMatchOptions( MQMO_MATCH_CORREL_ID);
	}

	for (int i = 0; i < iLoop; i++){
		if (!bClearAllMessages){
			// Only set correlid if !ClearAllMessages
			if (i==0){
				// First iteration - remove bootstrap messages
				pmd->setCorrelationId((MQBYTE*)pWorkSet->getBootstrapCorrelationId());
			}else{
				// Second iteration - remove volatile messages
				pmd->setCorrelationId((MQBYTE*)pWorkSet->getVolatileCorrelationId());
			}
		}

		for ( ;;){
			pWorkSet->getQueue()->get(*pmd,*pgmo);
			if (pWorkSet->getQueue()->reasonCode() == MQRC_NONE  ){
				continue;
			}else{
				if (pWorkSet->getQueue()->reasonCode() == MQRC_NO_MSG_AVAILABLE){
					break;
				}else{
					ostringstream	ost;
					ost << "MQGet for queue " << pWorkSet->getQName() << " failed with reason " << X_Mq::convertToStringMQRC(pWorkSet->getQueue()->reasonCode());
					throw X_Exception (E_ERROR,0,ost.str());
				}
			}
		}
	}

	printout(" ");
	if (bClearAllMessages){
		printout(pWorkSet->getQName() + " has been cleared of all messages");
	}else{
		printout(pWorkSet->getQName() + " has been cleared of relevant messages");
	}

}

//**********************************************************************************************************
// Browse message from State queue
//
// arg_cId							correlation id
// arg_pMd							MQ message object
//
// return							true if message available, false if no message available
//**********************************************************************************************************
bool	browseMessage(const char	arg_cId[MQ_MSG_ID_LENGTH],ImqMessage* arg_pMd){

	bool rc = true;
	
	ImqGetMessageOptions* pgmo = new ImqGetMessageOptions();
	
	// Match correlid in case multiple State queues are defined as aliases which resolve to the same local q (NOT RECOMMENDED)
	pgmo->setMatchOptions( MQMO_MATCH_CORREL_ID);

	pgmo->setOptions(MQGMO_BROWSE_NEXT);
	arg_pMd->setCorrelationId((MQBYTE*)arg_cId);

	pWorkSet->getQueue()->get(*arg_pMd,*pgmo);
	if (pWorkSet->getQueue()->reasonCode() == MQRC_NONE  ){
		;
	}else{
		if (pWorkSet->getQueue()->reasonCode() == MQRC_NO_MSG_AVAILABLE){
			rc = false;
		}else{
			ostringstream	ost;
			ost << "MQGet for queue " << pWorkSet->getQName() << " failed with reason " << X_Mq::convertToStringMQRC(pWorkSet->getQueue()->reasonCode());
			throw X_Exception (E_ERROR,0,ost.str());
		}
	}

	return rc;

}

//**********************************************************************************************************
// Load a state queue with bootstrap data
//
// pWorkSet							work properties
//**********************************************************************************************************
void	loadQueue(){
	char	szMessage[1000];
	pStateData->getStateDataAsString(szMessage);
	
	ImqPutMessageOptions* ppmo = new ImqPutMessageOptions();
	ImqMessage*	pmd = new ImqMessage();

	pmd->setPersistence(MQPER_PERSISTENT);

	pmd->setFormat(MQFMT_STRING);	

	pmd->setCorrelationId((MQBYTE*)pWorkSet->getBootstrapCorrelationId());

	pmd->useFullBuffer(szMessage,strlen(szMessage));

	pWorkSet->getQueue()->put(*pmd,*ppmo);
	if (pWorkSet->getQueue()->reasonCode() != MQRC_NONE  ){
		ostringstream	ost;
		ost << "MQPut failed for queue " << pWorkSet->getQName() << " with reason " << X_Mq::convertToStringMQRC(pWorkSet->getQueue()->reasonCode());
		throw X_Exception (E_ERROR,0,ost.str());
	}

	printout(pWorkSet->getQName() + " has been loaded ");
}

//**********************************************************************************************************
// Does what it says...
//**********************************************************************************************************
string	dottedOutput(string sValue){
	string s1 = sValue + MXC_DOTS;
	s1 = s1.substr(0,MXC_DOT_OFFSET);
	return s1;
}
	
//**********************************************************************************************************
// Help
//**********************************************************************************************************
void	printHelp(){
	cout << MXC_UTIL_PROG_IDENT << endl;
	cout << "This program loads and displays the State queue messages used by the maxconn security exit" << endl << endl;
	cout <<"Syntax for load: maxconnutil -l  -pf ParameterFile <-qm QmgrName>  <-s>" << endl;
	cout << "Syntax for display: maxconnutil -d  -sc ChannelName <-qm QmgrName>  <-q QueueName>" << endl;

}

void	printout(string arg_s){
	cout << arg_s << endl;
}

void	allVersions(){
	cout << MXC_UTIL_PROG_IDENT << endl;

	cout << "Component versions: "<< endl << endl;

	cout << "X_ArgumentParser - "<< X_ArgumentParser::getVersion() << endl;
	cout << "X_Exception - "<< X_Exception::getVersion() << endl;
	cout << "X_Mq - "<< X_Mq::getVersion() << endl;
	cout << "X_String - "<< X_String::getVersion() << endl;
	cout << "X_Tokenizer - "<< X_Tokenizer::getVersion() << endl;

}

