//***************************************************************************
//
//  V 1.0.1
//
// X_ArgumentParser accepts arguments and parses them.
// Arguments can be presented in main form (int,char**), as a string or in a file.
//
// Arguments can be parsed as any one of the following:
//				switches (first byte of switch must be '-'). Switches can have an associated value . 
//				name-value pairs
//				individual arguments 
//
//
// author	Tony Madden	(tm@mqware.com)
//
//***************************************************************************
// Fixes:
// V 1.0.1	make comment line value in a parameter file optional and variable
//          allow quote delimited values within set delimited files
//***************************************************************************
#include	<x_argumentparser.hpp>


X_ArgumentParser::X_ArgumentParser(long arg_l,bool arg_bComment){
	lParseType = arg_l;
	initializeValues();

	if (arg_bComment){
		cComment = _AP_COMMENT;
		bCommentsActivated = true;
	}else{
		bCommentsActivated = false;
	}

}

X_ArgumentParser::X_ArgumentParser(long arg_l,char	arg_cComment){
	lParseType = arg_l;
	initializeValues();
	cComment = arg_cComment;
	bCommentsActivated = true;

}


void	X_ArgumentParser::initializeValues(){
	//returnCode = 0;

	sParameters = " ";

	mNameValues.clear();

	cQuote = _AP_QUOTE;

	vOriginalArguments.resize(0);

	bSet = false;


}

//**********************************************************************
// Add arguments from a file
//**********************************************************************
void X_ArgumentParser::loadArgumentsFromFile(string arg_sFileName){

	vector<string> vect;
	
	internalFileLoad(arg_sFileName,NULL,&vect);

	appendStringVector(&vOriginalArguments,vect);

}

//**********************************************************************
// Add arguments from a file
//
// argsFileName		name of the file containing name/value pairs
// argsSetDelimiter	value of the set delimiter
//
//
// The name-values are grouped under Set Delimiter values, e.g.
//
// if the SetDelimiter value were 'Set:', then the file contents might look like this: 
/*
 Set: 
		Fld1  value1
		Fld2  value2
 Set: 
		Fld1  value3
		Fld3  value4	
*/
// Note that the Set Delimiter value (e.g. 'Set:') is repeated. The values under the tab (e.g. Fld1) do not have to exist in all tabs.
//
// The SetDelimiter is treated as the (repeating) key and all data under a SetDelimiter is treated as the text value
// When retrieving the data, use a get(...) method with the SetDelimiter value as key. X_ArgumentParser will return a text value which will
// contain all parameters under that instance of the SetDelimiter. Instantiate a separate X_ArgumentParser to parse the string.
//
// arg_sFileName		the file name
// arg_sSetDelimiter	the set delimiter
//**********************************************************************
void X_ArgumentParser::loadArgumentSetsFromFile(string arg_sFileName,string arg_sSetDelimiter){

	vector<string> vect;
	
	internalFileLoad(arg_sFileName,&arg_sSetDelimiter,&vect);

	appendStringVector(&vOriginalArguments,vect);
}

//**********************************************************************
// Add arguments from main type arguments
//**********************************************************************
void X_ArgumentParser::loadArgumentsFromMain(long arg_c, char ** arg_v)
{
	string sMethodName = "X_ArgumentParser::loadFromMainArguments";

	if (arg_c == 0){
		throw X_Exception (E_ERROR,0,"Error: No parameters",sMethodName);
	}

	vector<string> vs(arg_c-1);

	for (int i = 1;i < arg_c;i++){
		vs[i-1] = arg_v[i];
	}

	// Add the vector to the class vector vOriginalArguments
	appendStringVector(&vOriginalArguments,vs);


}

//**********************************************************************
// Add arguments from a string
//**********************************************************************
void X_ArgumentParser::loadArgumentsFromString(const string&	arg_sStringParms){

	// Add the argument string to the namevalue string
	vector<string> vect;
	internalLoadFromString(arg_sStringParms,&vect,cQuote);

	appendStringVector(&vOriginalArguments,vect);


}

//**********************************************************************
// Add contents of a vector to another vector
//
// arg_pTo			vector to be added to
// arg_From			vector which is read 
//**********************************************************************
void	X_ArgumentParser::appendStringVector(vector<string>* arg_pTo,const vector<string>& arg_From){
	long	lOriginalToSize = arg_pTo->size();
	arg_pTo->resize(arg_pTo->size() + arg_From.size());
	vector<string>::iterator ito;
	vector<string>::const_iterator ifrom;

	for (ifrom = arg_From.begin(),ito = arg_pTo->begin() + lOriginalToSize;ifrom != arg_From.end();ifrom++,ito++){
		*ito = *ifrom;
	}
}

//**********************************************************************
// Parse the arguments
//**********************************************************************
void	X_ArgumentParser::parse(){

	parse(true,false);
}

//**********************************************************************
// Parse the arguments
//
// argTrim			true if values are to be trimmed (before and after)
// argIgnoreCase	true if case of names (keys) is to be ignored 
//**********************************************************************
void	X_ArgumentParser::parse(bool arg_bTrim,bool arg_bIgnoreCase){

	bIgnoreCase = arg_bIgnoreCase;
	
	// Actually parse the stuff
	setNameValuePairs(arg_bTrim);

	bSet = true;
}

void	X_ArgumentParser::reset(){

	initializeValues();
}


//**********************************************************************
// Read argument file and get values
// This method concatenates the name value pairs into a string.
// File data must be character (i.e. non-printable characters are taken to be EOR or EOF)
//
// arg_sFileName		file name
// arg_psSetDelimiter	set delimiter  (NULL if parameters do not consitute sets)
// arg_pVector			vector of parsed arguments				
//**********************************************************************
void X_ArgumentParser::internalFileLoad(string	arg_sFileName, string* arg_psSetDelimiter,vector<string>*  arg_pVector){

	string sMethodName = "X_ArgumentParser::internalFileLoad";
	

	std::ifstream	parmFile(arg_sFileName.c_str());
	if (!parmFile || parmFile.fail()){
		ostringstream	ost;
		ost << "Error: file " << arg_sFileName << " could not be opened";
		throw X_Exception (E_ERROR,0,ost.str(),sMethodName);
	}

	char	c;
	bool	bComment = false;
	bool	bNewLine = true;

	string sArgs ;

	// Read the file into a string (sArgs)
	while (true){
		if (!parmFile.get(c)){
			if (parmFile.eof()){
				break;
			}else{
				ostringstream	ost;
				ost << "Error - read from " << arg_sFileName << " failed";
				throw X_Exception(E_ERROR,0,ost.str(),sMethodName);
			}
		}
		if (isspace(c)  ){
			if (c == '\n' || c == '\r'){
				bComment = false;
				bNewLine = true;
			}
			c = ' ';
		}else{
			if (bCommentsActivated && bNewLine && c == cComment ){
				bComment = true;
			}
			bNewLine = false;
		}

		if (bComment){
			continue;
		}

		sArgs += c ;
	}
	parmFile.close();

	// If a SetDelimiter is specified, then process data accordingly. 
	// The SetDelimiter is treated as the (repeating) key and all data under a SetDelimiter is treated as the value.
	// When retrieving the data, use one of the get() methods with the SetDelimiter as key. X_ArgumentParser will return the appropriate string which will
	// contain all parameters under that instance of the SetDelimiter. Then, instantiate a separate X_ArgumentParser to parse the string.
	// (The set delimiter is also described in method loadArgumentSetsFromFile() ).
	if (arg_psSetDelimiter != NULL){
		string::size_type stValueStart = 0;
		string::size_type stNewStart = 0;

		string	sData;

		// NOTE - any data prior to the first SetDelimiter is ignored
		stNewStart = sArgs.find(*arg_psSetDelimiter);
		if (stNewStart != string::npos){
			stValueStart = stNewStart + arg_psSetDelimiter->size();
			stNewStart++; 

			while (true){
				if (stNewStart > sArgs.size()){
					break;
				}
				stNewStart = sArgs.find(*arg_psSetDelimiter,stNewStart);
				if (stNewStart == string::npos){
					sData += " " + *arg_psSetDelimiter + " ";
					if (stValueStart  < sArgs.size()){
						// All data under a SetDelimiter instance is packaged in special delimiters and treated as a single value
						sData += (char)_AP_INTERNAL_SET_DELIMITER + sArgs.substr(stValueStart ) +(char)_AP_INTERNAL_SET_DELIMITER + " "; 
					}
					break;
				}

				// All data under a SetDelimiter instance is packaged in special delimiters and treated as a single value
				sData += " " + *arg_psSetDelimiter + (char)_AP_INTERNAL_SET_DELIMITER + sArgs.substr(stValueStart ,stNewStart - stValueStart) + (char)_AP_INTERNAL_SET_DELIMITER + " "; 
				stValueStart = stNewStart + arg_psSetDelimiter->size();
				stNewStart++;
			}
			// Add the set data to the parsed arguments
			internalLoadFromString(sData,arg_pVector,_AP_INTERNAL_SET_DELIMITER);
		}else{
			ostringstream	ost;
			ost << "Error - Parameter file " << arg_sFileName << " is empty";
			throw X_Exception(E_ERROR,0,ost.str(),sMethodName);

		}
	}else{
		internalLoadFromString(sArgs,arg_pVector,cQuote);
	}


}

//**********************************************************************
// Extract each space delimited value and add to the class variable vOriginalArguments
//
// arg_sStringArgs		string containing space delimited arguments
// arg_pVector			vector which will contain the parsed arguments
// arg_cDelimiter		value which specifies a collection of data which is not to be parsed (cf CDATA in XML)
//**********************************************************************
void	X_ArgumentParser::internalLoadFromString(const string& arg_sStringArgs,vector<string>*	arg_pVector,char	arg_cDelimiter){
	string sMethodName = "X_ArgumentParser::internalLoadFromString";

	string::size_type stStartDelimiter = 0;
	string::size_type stEndDelimiter = 0;

	string sRemainder = arg_sStringArgs;
	string sTemp;

	const vector<string>*	pVector;

	X_Tokenizer	tokenizer;

	while (true){
		// Search for delimited data. Data within a delimter are not treated as parameters.
		stStartDelimiter = sRemainder.find(arg_cDelimiter);
		if (stStartDelimiter == string::npos){
			// No quotes, so tokenize the remainder
			tokenizer.parse(sRemainder);
			pVector = tokenizer.getAllTokens();
			appendStringVector(arg_pVector,*pVector);
			break;
		}else{
			// Found a starting delimiter, so search for an end delimiter
			stEndDelimiter = sRemainder.find(arg_cDelimiter,stStartDelimiter+1);
			if (stEndDelimiter == string::npos){
				throw X_Exception(E_ERROR,0,"Error - incomplete quotes",sMethodName);
			}
			// Get the data prior to the start delimiter 
			sTemp = sRemainder.substr(0,stStartDelimiter);
			// Tokenize the data
			tokenizer.parse(sTemp);
			pVector = tokenizer.getAllTokens();
			appendStringVector(arg_pVector,*pVector);
			// Extract the data within the delimiters 
			sTemp = sRemainder.substr(stStartDelimiter+1,stEndDelimiter - stStartDelimiter -1);
			// Add it to the parsed arguments
			arg_pVector->push_back(sTemp);
			if (stEndDelimiter == sRemainder.size() - 1){
				break;
			}else{
				sRemainder = sRemainder.substr(stEndDelimiter+1);
			}
		}
	}

}

//**********************************************************************
// Populate a name value map (string,vector<string>)
//
// arg_bTrim		true if leading/trailing spaces are to be trimmed
//**********************************************************************
void X_ArgumentParser::setNameValuePairs(bool arg_bTrim){
	string sMethodName = "X_ArgumentParser::setNameValuePairs";

	X_StrVstrMap::iterator	itmap;

	string					sName;
	string					sValue;
	bool					bNoValue;

	mNameValues.clear();

	if (lParseType == _AP_NAME_VALUE_PAIRS && vOriginalArguments.size()%2 != 0){
		throw X_Exception (E_ERROR,0,"Error: invalid number of arguments (incomplete name value pairs)",sMethodName);
	}

	// _AP_SINGLES  means each argument is a separate entity  (i.e. the vector<string> component of mNameValues entry will be zero sized
	// _AP_SWITCHES means each argument is either a switch (first byte '-') or a value associated with a switch. Switches without values are allowed.
	// _AP_NAME_VALUE_PAIRS means each consecutive pair of arguments is a name/value pair
	for (vector<string>::iterator itorig = vOriginalArguments.begin();itorig != vOriginalArguments.end();itorig++){

		if (lParseType == _AP_SINGLES){
			bNoValue = true;
		}else{
			bNoValue = false;
		}
		sName = *itorig;
		if (lParseType == _AP_NAME_VALUE_PAIRS){
			sValue = *(++itorig);
		}else{
			if (lParseType == _AP_SWITCHES){
				if (sName[0] != _AP_SWITCH_PREFIX){
					ostringstream	ost;
					ost << "Error - Expected switch value, found " << sName;
					throw X_Exception (E_ERROR,0,ost.str(),sMethodName);

				}
				if ((itorig + 1) != vOriginalArguments.end() && (*(itorig+1))[0] != _AP_SWITCH_PREFIX){
					sValue = *(++itorig);
					bNoValue = false;
				}else{
					bNoValue = true;
				}
			}
		}

		// Trim, if necessary
		if (arg_bTrim && sValue.size()){
			sValue = X_String::trim(sValue);
		}

		if (bIgnoreCase){
			// All keys will be converted to lower case
			sName = X_String::toLower(sName);
		}
		itmap = mNameValues.find((const string &)sName);
		if (itmap == mNameValues.end()){
			// If map entry doesn't exist then add it
			if (bNoValue){
				mNameValues[sName] = NULL;
			}else{
				mNameValues[sName] = new vector<string>;
				mNameValues[sName]->push_back(sValue);
			}
			mNameValuesMetaData[sName] = new X_ArgumentParserMetaData;
		}else{
			if (bNoValue){
				if (itmap->second != NULL){
					ostringstream	ost;
					ost << "Error. Switch " << sName <<" cannot occur both with and without values. ";
					throw X_Exception (E_ERROR,0,ost.str(),sMethodName);
				}
			}else{
				// If map entry already exists then add the value to the second value (the string vector)
				if (itmap->second == NULL){
					ostringstream	ost;
					ost << "Error. Switch " << sName << " cannot occur both with and without values. ";
					throw X_Exception (E_ERROR,0,ost.str(),sMethodName);
				}else{
					(itmap->second)->push_back(sValue);
				}
			}
		}
		mNameValuesMetaData[sName]->incrementInstanceCount();
	}
}

//**********************************************************************
// Get the values associated with sName.
// Return > 0 only if the key exists AND has an associated value
//
// arg_sName					argument name
// arg_pvsValue					address of string vector into which value(s) will be inserted
// arg_bTrueOnlyIfValueExists	if true, then return > 0 only if name exists AND it has associated values
//								if false, then return > 0 if the name exists (regardless of whether there are any associated values)
//
// return						the number of instances within the values vector (or zero, see arg_bTrueOnlyIfValueExists)	 
//**********************************************************************
long	X_ArgumentParser::get(string	arg_sName,vector<string>* arg_pvsValue,bool arg_bTrueOnlyIfValueExists ){
	
	// get the value (the entire vector) for the specified key
	return getInternal(arg_sName,arg_pvsValue,arg_bTrueOnlyIfValueExists,0 );


}
//*******************************************************************************
// Get the value associated with sName.
// Return > 0 only if the key exists AND has an associated value
//
// arg_sName					argument name
// arg_psValue					address of string into which value will be inserted (i.e. element 0 of the value vector)
// arg_bTrueOnlyIfValueExists	if true, then return > 0 only if name exists AND it has associated values
//								if false, then return > 0 if the name exists (regardless of whether there are any associated values)
//
// return						the number of instances within the values vector (or perhaps zero, see arg_bTrueOnlyIfValueExists)	 
//*******************************************************************************
long	X_ArgumentParser::get(string	arg_sName ,string * arg_psValue,bool arg_bTrueOnlyIfValueExists ){

	vector<string> vs;
	long	lCount ;

	if (lCount = getInternal(arg_sName,&vs,arg_bTrueOnlyIfValueExists,1)){
		if (vs.size() > 0){
			*arg_psValue = vs[0];
			
		}else{
			*arg_psValue="";
		}
	}else{
		*arg_psValue="";
	}
	return lCount;

}

//*******************************************************************************
// Get the value associated with sName.
//
// arg_sName					argument name
// arg_psValue					address of string into which value will be inserted (i.e. element 0 of the value vector)
//
// return						the number of items within the vector	 
//*******************************************************************************
long	X_ArgumentParser::get(string	arg_sName ,string * arg_psValue){

	vector<string> vs;
	long	lCount;
	if (lCount = getInternal(arg_sName,&vs,false,1)){
		if (vs.size() > 0){
			*arg_psValue = vs[0];
			
		}else{
			*arg_psValue="";
		}
	}else{
		*arg_psValue="";
	}
	return lCount;
}

//*******************************************************************************
// Get the values associated with sName.
//
// arg_sName					argument name
// arg_pvsValue					address of string vector into which value(s) will be inserted
//
// return						the number of items within the vector	 
//*******************************************************************************
long	X_ArgumentParser::get(string	arg_sName,vector<string>* arg_pvsValue){
	return getInternal(arg_sName,arg_pvsValue,false,0);

}

//*******************************************************************************
// Get the value associated with the sName key (i.e. element 0 of the value vector)
//
// arg_sName						argument name
//
// return							the associated value or zero length string if key doesn't exist or has no value
//*******************************************************************************
string	X_ArgumentParser::get(string	arg_sName){
	vector<string> vs;

	X_ArgumentParser::getInternal(arg_sName,&vs,false,1);

	if (vs.size()){
		return vs[0];
	}else{
		return "";
	}
}

//*******************************************************************************
// Return the number of instances of the key
//
// arg_sName						argument name
// arg_lCount						the required number of instances (or zero for "any number of instances is valid"). 
//									If non-zero, then exception thrown if actual number does not match
//
// return							the number of instances within the values vector	 
//*******************************************************************************
long	X_ArgumentParser::numberOfInstances(string	arg_sName,long arg_lCount){

	return getInternal(arg_sName,0,false,arg_lCount);

}
//*******************************************************************************
long	X_ArgumentParser::numberOfInstances(string	arg_sName){

	return getInternal(arg_sName,0,false,0);

}

//*******************************************************************************
// Determine whether key exists
//
// arg_sName						argument name
//
// return							non-zero if key exists, zero if key does not exist.	 
//*******************************************************************************
long	X_ArgumentParser::exists(string	arg_sName){

	return getInternal(arg_sName,0,false,0);

}
//*******************************************************************************
// Get the name values as a map
//*******************************************************************************
void 		X_ArgumentParser::getNameValuesAsMap(X_StrVstrMap* arg_pMap){
	*arg_pMap = mNameValues;
}

//*******************************************************************************
// Get arguments as a vector
//*******************************************************************************
void		X_ArgumentParser::getArgumentsAsVector(vector<string>*	arg_pVector){
	*arg_pVector = vOriginalArguments;
}

//*******************************************************************************
// Use the key sName to get the string vector of values
//
//
// arg_sName							argument name
// arg_pvsValue							address of string vector into which value(s) will be inserted.
// arg_bProcessOnlyIfKeyAndValueExists	if true, then return true (and return the number of instances) only if name exists AND it has an associated value
//										if false, then return true if the name exists (regardless of whether there is an associated value).
//
// arg_lCount							If arg_lCount is non-zero then an exception is thrown if the number of occurances does not equal arg_lCount 

// returns								the number of instances of arg_sName (but see arg_bProcessOnlyIfKeyAndValueExists)
//*******************************************************************************
long	X_ArgumentParser::getInternal(string arg_sName,
									 vector<string>* arg_pValuesVector, bool  arg_bProcessOnlyIfKeyAndValueExists ,long arg_lCount){

	string sMethodName = "X_ArgumentParser::getInternal";

	bool	bRc;
	long	lCount = 0;

	if (arg_pValuesVector){
		arg_pValuesVector->resize(0);
	}

	if (mNameValues.size() == 0){
		if (bSet){
			bRc = false;
		}else{
			throw X_Exception (E_ERROR,0,"Error: Uninitialized X_ArgumentParser object",sMethodName);
		}
	}else{
		X_StrVstrMap::iterator it ;
		string					sName;

		if (bIgnoreCase){
			// All keys are lower case
			it = mNameValues.find(sName = X_String::toLower(arg_sName));
		}else{
			it = mNameValues.find(sName = arg_sName);
		}

		if (it == mNameValues.end()){
			bRc = false;
		}else{
			// Note the fact that the sName key value has been interrogated by incrementing reference count.
			mNameValuesMetaData[sName]->incrementReferenceCount();
			lCount = mNameValuesMetaData[sName]->getInstanceCount();
			if (it->second){
				if (arg_pValuesVector){
					*arg_pValuesVector  = *(it->second);
				}
				bRc = true;
			}else{
				if (arg_bProcessOnlyIfKeyAndValueExists ){
					bRc = false;
				}else{
					bRc = true;
				}
			}
		}
		
	}
	if (arg_lCount){
		if (bRc){
			if (lCount && arg_lCount != lCount ){
				ostringstream	ost;
				ost << "Error: parameter " << arg_sName << " occurs " << lCount << " times, but is required to occur " << arg_lCount << " times"; 
				throw X_Exception (E_ERROR,0,ost.str(),sMethodName);
			}
		}else{
			lCount = 0;
		}
	}

	return lCount;
}

//*******************************************************************************
// Set the quote value - default is double quote (")
//*******************************************************************************
void	X_ArgumentParser::setQuoteValue(char arg_cQ){
	cQuote = arg_cQ;
}

//*******************************************************************************
// Get the quote value
//*******************************************************************************
char	X_ArgumentParser::getQuoteValue(){
	return cQuote;
}

//*******************************************************************************
// For given key, get how many times it has been referenced (via a get() method)
//
// arg_sName		the key
// arg_plRefCount	the number of times the key has been referenced
//
// return			true if referenced
//*******************************************************************************
bool	X_ArgumentParser::getReferenceCount(string arg_sName,long*	arg_plRefCount){
	
	X_StrMetaMap::iterator it ;

	if (bIgnoreCase){
		// All keys are lower case
		it = mNameValuesMetaData.find(X_String::toLower(arg_sName));	
	}else{
		it = mNameValuesMetaData.find(arg_sName);	
	}

	bool bRc;

	if (it == mNameValuesMetaData.end()){
		*arg_plRefCount = 0;
		bRc = false;
	}else{
		*arg_plRefCount = (it->second)->getReferenceCount();
		bRc = true;
	}

	return bRc;
}
//*******************************************************************************
// For given key, get how many times it was specified in the arguments
//
// arg_sName		the key
// return			the number of times the key has been referenced
//*******************************************************************************
long	X_ArgumentParser::getInstanceCount(string arg_sName){
	long	lCount = 0;
	X_StrMetaMap::iterator it ;

	if (bIgnoreCase){
		// all keys are lower case
		it = mNameValuesMetaData.find(X_String::toLower(arg_sName));	
	}else{
		it = mNameValuesMetaData.find(arg_sName);	
	}

	if (it != mNameValuesMetaData.end()){
		lCount = (it->second)->getInstanceCount();
	}

	return lCount;
}

//*******************************************************************************
// Get all names with a zero reference count
//*******************************************************************************
bool	X_ArgumentParser::getUnusedNames(vector<string> * arg_pvsUnused){
	X_StrMetaMap::iterator it;
	bool	rc = false;

	if (arg_pvsUnused){
		arg_pvsUnused->resize(0);
	}
	
	// search the meta data object for all those keys which have not been interrogated
	// If required, populate a vector with all unused keys
	for (it = mNameValuesMetaData.begin(); it != mNameValuesMetaData.end();it++){
		if ((it->second)->getReferenceCount() == 0){
			rc = true;
			if (arg_pvsUnused){
				arg_pvsUnused->push_back(it->first);
			}else{
				break;
			}
		}
	}

	return rc;
}

//*******************************************************************************
// Are there names with a zero reference count
//*******************************************************************************
bool	X_ArgumentParser::isUnusedNames(){
	return getUnusedNames(0);
	
}

//*******************************************************************************
// String it up...
//*******************************************************************************
string	X_ArgumentParser::toString(){
	string s = " ";

	for (int i =0; i < vOriginalArguments.size();i++){
		s += vOriginalArguments[i] + " ";
	}

	return s;
}

string	X_ArgumentParser::getVersion(){
	return X_ARGUMENT_PARSER_VERSION;
}


//*******************************************************************************
// X_ArgumentParserMetaData is a utility class used by X_ArgumentParser
// It contains reference count info
//*******************************************************************************
X_ArgumentParserMetaData::X_ArgumentParserMetaData(){
	initVariables();
}


void	X_ArgumentParserMetaData::initVariables(){
	lReferenceCount = 0;
	lInstanceCount = 0;
}

bool	X_ArgumentParserMetaData::decrementReferenceCount(){
	bool	rc ;

	if (lReferenceCount){
		lReferenceCount--;
		rc = true;
	}else{
		rc = false;
	}
	return rc;
}

void	X_ArgumentParserMetaData::incrementReferenceCount(){
	lReferenceCount++;
}
void	X_ArgumentParserMetaData::incrementInstanceCount(){
	lInstanceCount++;
}
long	X_ArgumentParserMetaData::getReferenceCount(){
	return lReferenceCount;
}

long	X_ArgumentParserMetaData::getInstanceCount(){
	return lInstanceCount;
}

